
package de.codedo.conflicteditor;

public class Difference extends Object
{
    public final String key;
    public final Object currentValue;
    public final Object otherValue;

    @SuppressWarnings("hiding")
    public Difference(String key, Object currentValue, Object otherValue)
    {
        super();
        this.key = key;
        this.currentValue = currentValue;
        this.otherValue = otherValue;
    }

    @Override
    public String toString()
    {
        StringBuilder buf = new StringBuilder(128);
        buf.append(key);
        buf.append(" : ");
        buf.append(format(currentValue));
        buf.append(" - ");
        buf.append(format(otherValue));
        return buf.toString();
    }

    private String format(Object object)
    {
        if (object == null)
        {
            return null;
        }

        String string = object.toString();
        if (string.startsWith("\"") == false)
        {
            string = "\"" + string;
        }
        if (string.endsWith("\"") == false)
        {
            string = string + "\"";
        }
        return string;
    }

    @Override
    public int hashCode()
    {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((currentValue == null) ? 0 : currentValue.hashCode());
        result = prime * result + ((key == null) ? 0 : key.hashCode());
        result = prime * result + ((otherValue == null) ? 0 : otherValue.hashCode());
        return result;
    }

    @Override
    public boolean equals(Object obj)
    {
        if (this == obj)
        {
            return true;
        }
        if (obj == null)
        {
            return false;
        }
        if (getClass() != obj.getClass())
        {
            return false;
        }
        Difference other = (Difference)obj;
        if (currentValue == null)
        {
            if (other.currentValue != null)
            {
                return false;
            }
        }
        else if (!currentValue.equals(other.currentValue))
        {
            return false;
        }
        if (key == null)
        {
            if (other.key != null)
            {
                return false;
            }
        }
        else if (!key.equals(other.key))
        {
            return false;
        }
        if (otherValue == null)
        {
            if (other.otherValue != null)
            {
                return false;
            }
        }
        else if (!otherValue.equals(other.otherValue))
        {
            return false;
        }
        return true;
    }
}
