
package de.codedo.conflicteditor.gui;

import de.codedo.conflicteditor.Conflict;
import de.codedo.conflicteditor.ConflictsView;
import de.codedo.conflicteditor.CouchDb;
import de.codedo.conflicteditor.Difference;
import de.codedo.conflicteditor.DocumentMatcher;
import de.codedo.conflicteditor.UrlConnectionHttpAccess;

import java.io.IOException;
import java.io.Reader;
import java.net.URL;
import java.util.List;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.node.ObjectNode;

public class ConflictEditorController extends Object
{
    private CouchDb _database;
    private ObjectNode _currentConflict;
    private Difference _currentDifference;
    private String _conflictRevision;

    public JsonNode findConflicts(String dbUrl) throws IOException
    {
        _database = new CouchDb(dbUrl);
        ConflictsView conflictsView = new ConflictsView(_database);
        return conflictsView.getConflicts();
    }

    public List<Difference> differencesForConflict(JsonNode conflictNode) throws IOException
    {
        Conflict conflict = new Conflict(conflictNode);
        JsonNode currentDocument = conflict.getCurrentDocument();
        JsonNode conflictDocument = loadConflictingDocument(conflict);
        return new DocumentMatcher(currentDocument, conflictDocument).compare();
    }

    private JsonNode loadConflictingDocument(Conflict conflict) throws IOException
    {
        URL documentUrl = _database.urlForDocumentAndRevision(conflict.getId(),
            conflict.getConflictRevision());
        Reader reader = new UrlConnectionHttpAccess(documentUrl).get();
        return parseJson(reader);
    }

    private JsonNode parseJson(Reader reader) throws IOException
    {
        try
        {
            return new ObjectMapper().readTree(reader);
        }
        finally
        {
            reader.close();
        }
    }

    public void useCurrentValue()
    {
        JsonNode value = (JsonNode)_currentDifference.currentValue;
        _currentConflict.put(_currentDifference.key, value);
    }

    public void useConflictValue()
    {
        JsonNode value = (JsonNode)_currentDifference.otherValue;
        _currentConflict.put(_currentDifference.key, value);
    }

    public JsonNode getCurrentConflict()
    {
        return _currentConflict;
    }

    public void setCurrentConflict(JsonNode conflictNode) throws Exception
    {
        JsonNode node = conflictNode.get("key");
        JsonNode conflicts = node.get("_conflicts");
        if (conflicts.size() > 1)
        {
            throw new UnsupportedOperationException("there is more than one conflict");
        }
        _conflictRevision = conflicts.get(0).getValueAsText();

        _currentConflict = deepCopyWithoutConflict(node);
    }

    private ObjectNode deepCopyWithoutConflict(JsonNode node) throws Exception
    {
        // take a deep copy of the conflict node, we will be editing it here.
        String docString = node.toString();
        ObjectNode clonedNode = (ObjectNode)new ObjectMapper().readTree(docString);
        clonedNode.remove("_conflicts");
        return clonedNode;
    }

    public void setCurrentDifference(Difference difference)
    {
        _currentDifference = difference;
    }

    public void save() throws Exception
    {
        updateCurrentVersion();
        deleteConflictVersion();
    }

    private void updateCurrentVersion() throws Exception
    {
        String id = _currentConflict.get("_id").getValueAsText();
        URL updateUrl = _database.urlForDocument(id);
        Reader reader = new UrlConnectionHttpAccess(updateUrl).put(_currentConflict.toString());

        // TODO take care of the return value
        System.out.println(new ObjectMapper().readTree(reader));
    }

    private void deleteConflictVersion() throws Exception
    {
        String id = _currentConflict.get("_id").getValueAsText();
        URL deleteUrl = _database.urlForDocumentAndRevision(id, _conflictRevision);
        Reader reader = new UrlConnectionHttpAccess(deleteUrl).delete();

        // TODO take care of the return value
        System.out.println(new ObjectMapper().readTree(reader));
    }
}
