
package de.codedo.conflicteditor.gui;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableModel;

public class ConflictEditorFrame extends JFrame
{
    private JTextField _databaseUrlTextField;
    private JButton _findConflictsButton;
    private JList _conflictsList;
    private JTable _differencesTable;
    private ButtonPanel _buttonPanel;

    public ConflictEditorFrame() throws HeadlessException
    {
        super();
        setLayout(new GridBagLayout());
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setTitle("Conflict Editor");
        createGui();
    }

    private void createGui()
    {
        createDatabasePanel();
        createConflictsListAndDifferencesTable();
        createButtonPanel();
    }

    private void createDatabasePanel()
    {
        JPanel databasePanel = buildDatabasePanel();
        addDatabasePanel(databasePanel);
    }

    private JPanel buildDatabasePanel()
    {
        JPanel databasePanel = new JPanel();
        FlowLayout layout = new FlowLayout(FlowLayout.LEFT);
        databasePanel.setLayout(layout);

        JLabel label = new JLabel("Database URL:");
        databasePanel.add(label);

        _databaseUrlTextField = new JTextField(40);
        databasePanel.add(_databaseUrlTextField);

        _findConflictsButton = new JButton("Find Conflicts");
        databasePanel.add(_findConflictsButton);

        return databasePanel;
    }

    private void addDatabasePanel(JPanel databasePanel)
    {
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 0;
        constraints.weightx = 1;
        constraints.weighty = 0;
        constraints.anchor = GridBagConstraints.LINE_START;
        constraints.fill = GridBagConstraints.HORIZONTAL;
        getContentPane().add(databasePanel, constraints);
    }

    private void createConflictsListAndDifferencesTable()
    {
        JSplitPane splitPane = buildConflictsListAndDifferencesTableSplitPane();
        addConflictsListAndDifferencesTableSplitPane(splitPane);
    }

    private JSplitPane buildConflictsListAndDifferencesTableSplitPane()
    {
        JScrollPane conflictsScrollPane = buildConflictsListScrollPane();
        JScrollPane differencesScrollPane = buildDifferencesScrollPane();

        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitPane.add(conflictsScrollPane);
        splitPane.add(differencesScrollPane);
        return splitPane;
    }

    private JScrollPane buildConflictsListScrollPane()
    {
        buildConflictsList();
        return new JScrollPane(_conflictsList);
    }

    private void buildConflictsList()
    {
        _conflictsList = new JList();
    }

    private JScrollPane buildDifferencesScrollPane()
    {
        buildDifferencesTable();
        return new JScrollPane(_differencesTable);
    }

    private void buildDifferencesTable()
    {
        _differencesTable = new JTable();
        _differencesTable.setDefaultRenderer(Object.class, new SpecialCharactersTableCellRenderer());
        _differencesTable.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        TableModel model = new DifferenceTableModel();
        _differencesTable.setModel(model);
    }

    private void addConflictsListAndDifferencesTableSplitPane(JSplitPane splitPane)
    {
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 1;
        constraints.weightx = 1;
        constraints.weighty = 0.9;
        constraints.fill = GridBagConstraints.BOTH;
        getContentPane().add(splitPane, constraints);
    }

    private void createButtonPanel()
    {
        _buttonPanel = new ButtonPanel();
        addButtonPanel(_buttonPanel);
    }

    private void addButtonPanel(JPanel buttonPanel)
    {
        GridBagConstraints constraints = new GridBagConstraints();
        constraints.gridx = 0;
        constraints.gridy = 2;
        getContentPane().add(buttonPanel, constraints);
    }

    public void disableButtons()
    {
        _buttonPanel.disableButtons();
    }

    public void enableButtons()
    {
        _buttonPanel.enableButtons();
    }

    public JTextField getDatabaseUrlTextField()
    {
        return _databaseUrlTextField;
    }

    public JButton getFindConflictsButton()
    {
        return _findConflictsButton;
    }

    public JList getConflictsList()
    {
        return _conflictsList;
    }

    public JTable getDifferencesTable()
    {
        return _differencesTable;
    }

    public JButton getCurrentValueButton()
    {
        return _buttonPanel.getCurrentValueButton();
    }

    public JButton getConflictValueButton()
    {
        return _buttonPanel.getConflictValueButton();
    }

    public JButton getPreviewButton()
    {
        return _buttonPanel.getPreviewButton();
    }

    public JButton getSaveButton()
    {
        return _buttonPanel.getSaveButton();
    }
}
